import os
import json
import requests
import time
import logging
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException, TimeoutException, StaleElementReferenceException
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.common.exceptions import SessionNotCreatedException
try:
    from IMDBTraktSyncer import checkChromedriver
    from IMDBTraktSyncer import verifyCredentials as VC
    from IMDBTraktSyncer import traktData
    from IMDBTraktSyncer import imdbData
    from IMDBTraktSyncer import errorHandling as EH
except ImportError:
    import checkChromedriver
    import verifyCredentials as VC
    import traktData
    import imdbData
    import errorHandling as EH
from chromedriver_py import binary_path


def main():
    try:

        #Get credentials
        imdb_username = VC.imdb_username
        imdb_password = VC.imdb_password
        
        directory = os.path.dirname(os.path.realpath(__file__))
        
        logging.getLogger('selenium.webdriver').setLevel(logging.WARNING)
        
        #Start web driver
        print('Starting webdriver...')
        options = Options()
        options.add_argument("--headless=new")
        options.add_argument('--user-agent=Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.110 Safari/537.3')
        options.add_experimental_option("prefs", {"download.default_directory": directory, "download.directory_upgrade": True, "download.prompt_for_download": False, "credentials_enable_service": False, "profile.password_manager_enabled": False})
        options.add_argument("--disable-save-password-bubble")
        options.add_argument("--disable-infobars")
        options.add_argument("--disable-autofill-for-password-fields")
        options.add_argument('--disable-notifications')
        options.add_argument("--disable-third-party-cookies")
        options.add_argument("--disable-dev-shm-usage")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-extensions")
        options.add_experimental_option("excludeSwitches", ["enable-automation", "enable-logging"])
        options.add_experimental_option("useAutomationExtension", False)
        options.add_argument('--log-level=3')

        service = Service(executable_path=binary_path)
        try:
            driver = webdriver.Chrome(service=service, options=options)
        except SessionNotCreatedException as e:
            error_message = str(e)
            if "This version of ChromeDriver only supports Chrome version" in error_message:
                extract_message = error_message.split("Stacktrace:")[0].replace("Message: session not created:", "").strip()
                print(f"Error: {extract_message}")
                print("See this link for details on how to fix: https://github.com/RileyXX/IMDB-Trakt-Syncer/issues/16")
                raise SystemExit
            else:
                raise

        wait = WebDriverWait(driver, 10)

        driver.get('https://www.imdb.com/registration/signin')

        # wait for sign in link to appear and then click it
        sign_in_link = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, 'a.list-group-item > .auth-provider-text')))
        if 'IMDb' in sign_in_link.text:
            sign_in_link.click()

        # wait for email input field and password input field to appear, then enter credentials and submit
        email_input = wait.until(EC.presence_of_all_elements_located((By.CSS_SELECTOR, "input[type='email']")))[0]
        password_input = wait.until(EC.presence_of_all_elements_located((By.CSS_SELECTOR, "input[type='password']")))[0]
        email_input.send_keys(imdb_username)
        password_input.send_keys(imdb_password)
        submit_button = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, "input[type='submit']")))
        submit_button.click()

        time.sleep(2)

        # go to IMDB homepage
        driver.get('https://www.imdb.com/')

        time.sleep(2)

        # Check if signed in
        element = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, ".nav__userMenu.navbar__user")))
        if element.find_elements(By.CSS_SELECTOR, ".imdb-header__account-toggle--logged-in"):
            print("Successfully signed in to IMDB")
        else:
            print("Error: Not signed in to IMDB")
            print("\nPossible IMDB captcha check or IMDB login incorrect.")
            print("\nIf your login is correct then an IMDB captcha check likely the cause of this error. To fix this, simply login to the IMDB website in your browser, preferably Chrome, and from the same computer. If logged in, log out and log back in. It may ask you to fill in a captcha; complete it and finish logging in. After logging in successfully on your browser, run the script again and it should work. You may need to repeat this step once or twice if it still gives you issues.")
            print("\nIf your IMDB login is incorrect, simply edit the credentials.txt file with the correct login or delete the credentials file and run the script again.")
            print("\nSee this GitHub link for more details: https://github.com/RileyXX/IMDB-Trakt-Syncer/issues/2")
            print("\nStopping script...")
            raise SystemExit
        
        trakt_watchlist, trakt_ratings, trakt_reviews = traktData.getTraktData()
        imdb_watchlist, imdb_ratings, imdb_reviews = imdbData.getImdbData(imdb_username, imdb_password, driver, directory, wait)

        #Get trakt and imdb ratings and filter out trakt ratings with missing imdb id
        trakt_ratings = [rating for rating in trakt_ratings if rating['ID'] is not None]
        imdb_ratings = [rating for rating in imdb_ratings if rating['ID'] is not None]
        trakt_reviews = [review for review in trakt_reviews if review['ID'] is not None]
        imdb_reviews = [review for review in imdb_reviews if review['ID'] is not None]
        trakt_watchlist = [item for item in trakt_watchlist if item['ID'] is not None]
        imdb_watchlist = [item for item in imdb_watchlist if item['ID'] is not None]
        #Filter out ratings already set
        imdb_ratings_to_set = [rating for rating in trakt_ratings if rating['ID'] not in [imdb_rating['ID'] for imdb_rating in imdb_ratings]]
        trakt_ratings_to_set = [rating for rating in imdb_ratings if rating['ID'] not in [trakt_rating['ID'] for trakt_rating in trakt_ratings]]
        imdb_reviews_to_set = [review for review in trakt_reviews if review['ID'] not in [imdb_review['ID'] for imdb_review in imdb_reviews]]
        trakt_reviews_to_set = [review for review in imdb_reviews if review['ID'] not in [trakt_review['ID'] for trakt_review in trakt_reviews]]
        imdb_watchlist_to_set = [item for item in trakt_watchlist if item['ID'] not in [imdb_item['ID'] for imdb_item in imdb_watchlist]]
        trakt_watchlist_to_set = [item for item in imdb_watchlist if item['ID'] not in [trakt_item['ID'] for trakt_item in trakt_watchlist]]
        # Remove duplicate reviews and filter by out any items for imdb_reviews_to_set where comment length is less than 600 characters
        def remove_duplicates_and_filter(lst, key, min_comment_length=None):
            seen = set()
            result = []
            for item in lst:
                if item[key] not in seen and (min_comment_length is None or ('Comment' in item and len(item['Comment']) >= min_comment_length)):
                    seen.add(item[key])
                    result.append(item)
            return result

        imdb_reviews_to_set = remove_duplicates_and_filter(imdb_reviews_to_set, 'ID', 600)
        trakt_reviews_to_set = remove_duplicates_and_filter(trakt_reviews_to_set, 'ID')
        
        # If sync_watchlist_value is true
        if VC.sync_watchlist_value:
            # Set Trakt Watchlist Items
            if trakt_watchlist_to_set:
                print('Setting Trakt Watchlist Items')

                # Count the total number of items
                num_items = len(trakt_watchlist_to_set)
                item_count = 0

                for item in trakt_watchlist_to_set:
                    item_count += 1
                    imdb_id = item['ID']
                    media_type = item['Type']  # 'movie', 'show', or 'episode'

                    url = f"https://api.trakt.tv/sync/watchlist"

                    data = {
                        "movies": [],
                        "shows": [],
                        "episodes": []
                    }

                    if media_type == 'movie':
                        data['movies'].append({
                            "ids": {
                                "imdb": imdb_id
                            }
                        })
                    elif media_type == 'show':
                        data['shows'].append({
                            "ids": {
                                "imdb": imdb_id
                            }
                        })
                    elif media_type == 'episode':
                        data['episodes'].append({
                            "ids": {
                                "imdb": imdb_id
                            }
                        })

                    response = EH.make_trakt_request(url, payload=data)
                    if response:
                        print(f"Adding item ({item_count} of {num_items}): {item['Title']} ({item['Year']}) to Trakt Watchlist")
                    else:
                        print(f"Failed to add item ({item_count} of {num_items}): {item['Title']} ({item['Year']}) to Trakt Watchlist")
                        print("Error Response:", response.content, response.status_code)

                print('Trakt Watchlist Items Set Successfully')
            else:
                print('No Trakt Watchlist Items To Set')

            # Set IMDB Watchlist Items
            if imdb_watchlist_to_set:
                print('Setting IMDB Watchlist Items')
                
                # Count the total number of items
                num_items = len(imdb_watchlist_to_set)
                item_count = 0
                
                for item in imdb_watchlist_to_set:
                    try:
                        item_count += 1
                        year_str = f' ({item["Year"]})' if item["Year"] is not None else '' # sometimes year is None for episodes from trakt so remove it from the print string
                        print(f"Adding item ({item_count} of {num_items}): {item['Title']}{year_str} to IMDB Watchlist")
                        
                        driver.get(f'https://www.imdb.com/title/{item["ID"]}/')
                                            
                        watchlist_button = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, 'button[data-testid="tm-box-wl-button"]')))
                        
                        # Check if item is already in watchlist otherwise skip it
                        if 'ipc-icon--done' not in watchlist_button.get_attribute('innerHTML'):
                            watchlist_button.click()
                            time.sleep(1)
                    except (NoSuchElementException, TimeoutException):
                        print(f"Failed to add item ({item_count} of {num_items}): {item['Title']}{year_str} to IMDB Watchlist ({item['ID']})")
                        pass

                
                print('Setting IMDB Watchlist Items Complete')
            else:
                print('No IMDB Watchlist Items To Set')

        #Set Trakt Ratings
        if trakt_ratings_to_set:
            print('Setting Trakt Ratings')

            # Set the API endpoints
            rate_url = "https://api.trakt.tv/sync/ratings"
            
            # Count the total number of items
            num_items = len(trakt_ratings_to_set)
            item_count = 0
                    
            # Loop through your data table and rate each item on Trakt
            for item in trakt_ratings_to_set:
                item_count += 1
                if item["Type"] == "show":
                    # This is a TV show
                    data = {
                        "shows": [{
                            "ids": {
                                "imdb": item["ID"]
                            },
                            "rating": item["Rating"]
                        }]
                    }
                    print(f"Rating TV show ({item_count} of {num_items}): {item['Title']} ({item['Year']}): {item['Rating']}/10 on Trakt")
                elif item["Type"] == "movie":
                    # This is a movie
                    data = {
                        "movies": [{
                            "ids": {
                                "imdb": item["ID"]
                            },
                            "rating": item["Rating"]
                        }]
                    }
                    print(f"Rating movie ({item_count} of {num_items}): {item['Title']} ({item['Year']}): {item['Rating']}/10 on Trakt")
                elif item["Type"] == "episode":
                    # This is an episode
                    data = {
                        "episodes": [{
                            "ids": {
                                "imdb": item["ID"]
                            },
                            "rating": item["Rating"]
                        }]
                    }
                    print(f"Rating episode ({item_count} of {num_items}): {item['Title']} ({item['Year']}): {item['Rating']}/10 on Trakt")

                # Make the API call to rate the item
                response = EH.make_trakt_request(rate_url, payload=data)

                if response is None:
                    print(f"Error rating {item}: {response.content}")

            print('Setting Trakt Ratings Complete')
        else:
            print('No Trakt Ratings To Set')

        #Set IMDB Ratings
        if imdb_ratings_to_set:
            print('Setting IMDB Ratings')

            # loop through each movie and TV show rating and submit rating on IMDB website
            for i, item in enumerate(imdb_ratings_to_set, 1):
                year_str = f' ({item["Year"]})' if item["Year"] is not None else '' # sometimes year is None for episodes from trakt so remove it from the print string
                print(f'Rating {item["Type"]}: ({i} of {len(imdb_ratings_to_set)}) {item["Title"]}{year_str}: {item["Rating"]}/10 on IMDB')
                driver.get(f'https://www.imdb.com/title/{item["ID"]}/')

                # click on "Rate" button and select rating option, then submit rating
                button = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, 'button.ipc-btn span')))
                try:
                    element_rating_bar = button.find_element(By.CSS_SELECTOR, '[data-testid="hero-rating-bar__user-rating__unrated"]')
                    if element_rating_bar:
                        driver.execute_script("arguments[0].click();", button)
                        rating_option_element = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, f'button[aria-label="Rate {item["Rating"]}"]')))
                        driver.execute_script("arguments[0].click();", rating_option_element)
                        submit_element = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, 'button.ipc-rating-prompt__rate-button')))
                        submit_element.click()
                        time.sleep(1)
                except (NoSuchElementException, TimeoutException):
                    print(f'Failed to rate {item["Type"]}: ({i} of {len(imdb_ratings_to_set)}) {item["Title"]}{year_str}: {item["Rating"]}/10 on IMDB ({item["ID"]})')
                    pass

            print('Setting IMDB Ratings Complete')
        else:
            print('No IMDB Ratings To Set')

        # If sync_reviews_value is true
        if VC.sync_reviews_value:
            # Set Trakt Reviews
            if trakt_reviews_to_set:
                print('Setting Trakt Reviews')

                # Count the total number of items
                num_items = len(trakt_reviews_to_set)
                item_count = 0

                for review in trakt_reviews_to_set:
                    item_count += 1
                    imdb_id = review['ID']
                    comment = review['Comment']
                    media_type = review['Type']  # 'movie', 'show', or 'episode'

                    url = f"https://api.trakt.tv/comments"

                    data = {
                        "comment": comment
                    }

                    if media_type == 'movie':
                        data['movie'] = {
                            "ids": {
                                "imdb": imdb_id
                            }
                        }
                    elif media_type == 'show':
                        data['show'] = {
                            "ids": {
                                "imdb": imdb_id
                            }
                        }
                    elif media_type == 'episode':
                        data['episode'] = {
                            "ids": {
                                "imdb": episode_id
                            }
                        }
                    
                    response = EH.make_trakt_request(url, payload=data)
                    if response:
                        print(f"Submitted comment ({item_count} of {num_items}): {review['Title']} ({review['Year']}) on Trakt")
                    else:
                        print(f"Failed to submit comment ({item_count} of {num_items}): {review['Title']} ({review['Year']}) on Trakt")
                        print("Error Response:", response.content, response.status_code)

                print('Trakt Reviews Set Successfully')
            else:
                print('No Trakt Reviews To Set')

            # Set IMDB Reviews
            if imdb_reviews_to_set:
                # Call the check_last_run() function
                if VC.check_imdb_reviews_last_submitted():
                    print('Setting IMDB Reviews')
                    
                    # Count the total number of items
                    num_items = len(trakt_reviews_to_set)
                    item_count = 0
                    
                    for review in imdb_reviews_to_set:
                        item_count += 1
                        try:
                            print(f"Submitting review ({item_count} of {num_items}): {review['Title']} ({review['Year']}) on IMDB")
                            driver.get(f'https://contribute.imdb.com/review/{review["ID"]}/add?bus=imdb')
                            
                            review_title_input = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, "input.klondike-input")))
                            review_input = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, "textarea.klondike-textarea")))
                            
                            review_title_input.send_keys("My Review")
                            review_input.send_keys(review["Comment"])
                            
                            no_element = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, "ul.klondike-userreview-spoiler li:nth-child(2)")))
                            yes_element = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, "ul.klondike-userreview-spoiler li:nth-child(1)")))
                            
                            if review["Spoiler"]:
                                yes_element.click()                        
                            else:
                                no_element.click()
                                                    
                            submit_button = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, "input.a-button-input[type='submit']")))

                            submit_button.click()
                            
                            time.sleep(1)
                        except (NoSuchElementException, TimeoutException):
                            print(f"Failed to submit review ({item_count} of {num_items}): {review['Title']} ({review['Year']}) on IMDB ({item['ID']})")
                            pass
                    
                    print('Setting IMDB Reviews Complete')
                else:
                    print('IMDB reviews were submitted within the last 7 days. Skipping IMDB review submission.')
            else:
                print('No IMDB Reviews To Set')

        #Close web driver
        print("Closing webdriver...")
        driver.quit()
        service.stop()
    
    except Exception as e:
        error_message = "An error occurred while running the script."
        EH.report_error(error_message)

if __name__ == '__main__':
    main()


import requests
try:
    from IMDBTraktSyncer import errorHandling as EH
except ImportError:
    import errorHandling as EH

def authenticate(client_id, client_secret):
    CLIENT_ID = client_id
    CLIENT_SECRET = client_secret
    REDIRECT_URI = 'urn:ietf:wg:oauth:2.0:oob'

    # Set up the authorization endpoint URL
    auth_url = 'https://trakt.tv/oauth/authorize'

    # Construct the authorization URL with the necessary parameters
    params = {
        'response_type': 'code',
        'client_id': CLIENT_ID,
        'redirect_uri': REDIRECT_URI,
        'state': 'mystate', # Optional - used for CSRF protection
    }
    auth_url += '?' + '&'.join([f'{key}={value}' for key, value in params.items()])

    # Print out the authorization URL and instruct the user to visit it
    print(f'\nPlease visit the following URL to authorize this application: \n{auth_url}\n')

    # After the user grants authorization, they will be redirected back to the redirect URI with a temporary authorization code.
    # Extract the authorization code from the URL and use it to request an access token from the Trakt API.
    authorization_code = input('Please enter the authorization code from the URL: ')

    # Set up the access token request
    headers = {
        'Content-Type': 'application/json'
    }
    data = {
        'code': authorization_code,
        'client_id': CLIENT_ID,
        'client_secret': CLIENT_SECRET,
        'redirect_uri': REDIRECT_URI,
        'grant_type': 'authorization_code'
    }

    # Make the request to get the access token
    response = EH.make_trakt_request('https://api.trakt.tv/oauth/token', payload=data)

    # Parse the JSON response from the API
    json_data = response.json()

    # Extract the access token from the response
    ACCESS_TOKEN = json_data['access_token']
    
    return ACCESS_TOKEN


import os
import json
import datetime
try:
    from IMDBTraktSyncer import authTrakt
except ImportError:
    import authTrakt

# Define the file path
here = os.path.abspath(os.path.dirname(__file__))
file_path = os.path.join(here, 'credentials.txt')

# Old version support (v1.0.6 and below). Convert old credentials.txt format to json
if os.path.isfile(file_path):
    with open(file_path, 'r') as txt_file:
        lines = txt_file.readlines()
    if lines[0].startswith("trakt_client_id="):
        values = {}
        for line in lines:
            key, value = line.strip().split("=")
            values[key] = value
        with open(file_path, 'w') as txt_file:
            json.dump(values, txt_file)
        print("Warning: You are using a depreciated credentials.txt file.\nConversion successful: credentials.txt file converted to the new JSON format.")

# Check if the file exists
if not os.path.isfile(file_path):
    # If the file does not exist, create it with default values
    default_values = {
        "trakt_client_id": "empty",
        "trakt_client_secret": "empty",
        "trakt_access_token": "empty",
        "imdb_username": "empty",
        "imdb_password": "empty"
    }
    with open(file_path, "w") as f:
        json.dump(default_values, f)

# Load the values from the file
with open(file_path, "r") as f:
    values = json.load(f)

# Check if any of the values are "empty" and prompt the user to enter them
for key in values.keys():
    if values[key] == "empty" and key != "trakt_access_token":
        values[key] = input(f"Please enter a value for {key}: ").strip()
        with open(file_path, "w") as f:
            json.dump(values, f)

# Get the trakt_access_token value if it exists, or run the authTrakt.py function to get it
trakt_access_token = None
if "trakt_access_token" in values and values["trakt_access_token"] != "empty":
    trakt_access_token = values["trakt_access_token"]
else:
    client_id = values["trakt_client_id"]
    client_secret = values["trakt_client_secret"]
    trakt_access_token = authTrakt.authenticate(client_id, client_secret)
    values["trakt_access_token"] = trakt_access_token
    with open(file_path, "w") as f:
        json.dump(values, f)


def prompt_sync_watchlist():
    # Define the file path
    here = os.path.abspath(os.path.dirname(__file__))
    file_path = os.path.join(here, 'credentials.txt')

    # Check if credentials file exists
    try:
        with open(file_path, 'r') as file:
            credentials = json.load(file)
            sync_watchlist_value = credentials.get('sync_watchlist')
            if sync_watchlist_value is not None:
                return sync_watchlist_value
    except FileNotFoundError:
        pass

    while True:
        # Prompt the user for input
        print("Do you want to sync watchlists? (y/n)")
        user_input = input("Enter your choice: ")

        # Validate user input
        if user_input.lower() == 'y':
            sync_watchlist_value = True
            break
        elif user_input.lower() == 'n':
            sync_watchlist_value = False
            break
        else:
            # Invalid input, ask again
            print("Invalid input. Please enter 'y' or 'n'.")

    # Update the value in the JSON file
    credentials = {}
    try:
        with open(file_path, 'r') as file:
            credentials = json.load(file)
    except FileNotFoundError:
        pass

    credentials['sync_watchlist'] = sync_watchlist_value

    with open(file_path, 'w') as file:
        json.dump(credentials, file)

    # return true or false
    return sync_watchlist_value
sync_watchlist_value = prompt_sync_watchlist()

# Last run function, used to determine when the last time IMDB reviews were submitted    
def check_imdb_reviews_last_submitted():
    # Load credentials from credentials.txt
    if os.path.exists(file_path):
        with open(file_path, 'r') as file:
            credentials = json.load(file)
    else:
        credentials = {}

    imdb_reviews_last_submitted_date_str = credentials.get('imdb_reviews_last_submitted_date')

    # If imdb_reviews_last_submitted_date is not available or not in the correct format, consider it as 7 days ago
    imdb_reviews_last_submitted_date = datetime.datetime.strptime(imdb_reviews_last_submitted_date_str, '%Y-%m-%d %H:%M:%S') if imdb_reviews_last_submitted_date_str else datetime.datetime.now() - datetime.timedelta(hours=168)

    # Check if 168 hours have passed since the last run
    if datetime.datetime.now() - imdb_reviews_last_submitted_date >= datetime.timedelta(hours=168):
        # Update the imdb_reviews_last_submitted_date with the current time
        credentials['imdb_reviews_last_submitted_date'] = datetime.datetime.now().strftime('%Y-%m-%d %H:%M:%S')

        # Save updated credentials to credentials.txt
        with open(file_path, 'w') as file:
            json.dump(credentials, file)

        return True
    else:
        return False
        
def prompt_sync_reviews():
    # Define the file path
    here = os.path.abspath(os.path.dirname(__file__))
    file_path = os.path.join(here, 'credentials.txt')

    # Check if credentials file exists
    try:
        with open(file_path, 'r') as file:
            credentials = json.load(file)
            sync_reviews_value = credentials.get('sync_reviews')
            if sync_reviews_value is not None:
                return sync_reviews_value
    except FileNotFoundError:
        pass

    while True:
        # Prompt the user for input
        print("Ratings are synced by default. Please note: comments synced to IMDB will use \"My Review\" as the title field.")
        print("Do you want to sync comments? (y/n)")
        user_input = input("Enter your choice: ")

        # Validate user input
        if user_input.lower() == 'y':
            sync_reviews_value = True
            break
        elif user_input.lower() == 'n':
            sync_reviews_value = False
            break
        else:
            # Invalid input, ask again
            print("Invalid input. Please enter 'y' or 'n'.")

    # Update the value in the JSON file
    credentials = {}
    try:
        with open(file_path, 'r') as file:
            credentials = json.load(file)
    except FileNotFoundError:
        pass

    credentials['sync_reviews'] = sync_reviews_value

    with open(file_path, 'w') as file:
        json.dump(credentials, file)

    # return true or false
    return sync_reviews_value
sync_reviews_value = prompt_sync_reviews()


# Save the credential values as variables
trakt_client_id = values["trakt_client_id"]
trakt_client_secret = values["trakt_client_secret"]
trakt_access_token = values["trakt_access_token"]
imdb_username = values["imdb_username"]
imdb_password = values["imdb_password"]


import os
import time
import csv
import requests
from selenium import webdriver
from selenium.webdriver.chrome.service import Service
from selenium.webdriver.common.by import By
from selenium.common.exceptions import NoSuchElementException, TimeoutException, StaleElementReferenceException
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from chromedriver_py import binary_path
try:
    from IMDBTraktSyncer import errorHandling as EH
except ImportError:
    import errorHandling as EH

def getImdbData(imdb_username, imdb_password, driver, directory, wait):
    # Process IMDB Ratings Reviews & Watchlist
    print('Processing IMDB Data')
    
    #Get IMDB Watchlist Items
    try:
        driver.get('https://www.imdb.com/list/watchlist')

        csv_link = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, ".export a")))
        csv_link.click()

        #Wait for csv download to complete
        time.sleep(8)

        imdb_watchlist = []

        # Read the watchlist items from the CSV file
        here = os.path.abspath(os.path.dirname(__file__))
        watchlist_path = os.path.join(here, 'WATCHLIST.csv')
        try:
            with open(watchlist_path, 'r') as file:
                reader = csv.reader(file)
                header = next(reader)  # skip the header row
                for row in reader:
                    title = row[5]
                    year = row[10]
                    imdb_id = row[1]
                    if "tvSeries" in row[7] or "tvMiniSeries" in row[7]:
                        type = "show"
                    elif "tvEpisode" in row[7]:
                        type = "episode"
                    elif "movie" in row[7]:
                        type = "movie"
                    else:
                        type = "unknown"
                    imdb_watchlist.append({'Title': title, 'Year': year, 'ID': imdb_id, 'Type': type})
        except FileNotFoundError:
            print('Ratings file not found')
            
        # Delete csv files
        for file in os.listdir(directory):
            if file.endswith('.csv') and 'WATCHLIST' in file:
                os.remove(os.path.join(directory, file))

    except (NoSuchElementException, TimeoutException):
        # No IMDB Watchlist Items
        imdb_watchlist = []
        pass
    
    # Get IMDB Ratings
    try:
        driver.get('https://www.imdb.com/list/ratings')

        dropdown = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, ".circle")))
        dropdown.click()

        csv_link = wait.until(EC.presence_of_element_located((By.CSS_SELECTOR, ".pop-up-menu-list-items a.pop-up-menu-list-item-link")))
        csv_link.click()

        #Wait for csv download to complete
        time.sleep(8)

        imdb_ratings = []

        # Read the ratings from the CSV file
        here = os.path.abspath(os.path.dirname(__file__))
        ratings_path = os.path.join(here, 'ratings.csv')
        try:
            with open(ratings_path, 'r') as file:
                reader = csv.reader(file)
                header = next(reader)  # skip the header row
                for row in reader:
                    title = row[3]
                    year = row[8]
                    rating = row[1]
                    imdb_id = row[0]
                    if "tvSeries" in row[5] or "tvMiniSeries" in row[5]:
                        type = "show"
                    elif "tvEpisode" in row[5]:
                        type = "episode"
                    elif "movie" in row[5]:
                        type = "movie"
                    else:
                        type = "unknown"
                    imdb_ratings.append({'Title': title, 'Year': year, 'Rating': rating, 'ID': imdb_id, 'Type': type})
        except FileNotFoundError:
            print('Ratings file not found')
            
        # Delete csv files
        for file in os.listdir(directory):
            if file.endswith('.csv') and 'ratings' in file:
                os.remove(os.path.join(directory, file))
    except (NoSuchElementException, TimeoutException):
        # No IMDB Ratings
        imdb_ratings = []
        pass
            
    def get_media_type(imdb_id):
        url = f"https://api.trakt.tv/search/imdb/{imdb_id}"
        response = EH.make_trakt_request(url)
        if response:
            results = response.json()
            if results:
                media_type = results[0].get('type')
                return media_type
        return None

    #Get IMDB Reviews
    driver.get('https://www.imdb.com/profile')
    reviews_link = driver.find_element(By.CSS_SELECTOR, "div.aux-content-widget-2 div.subNavItem a[href*='comments-index']")
    reviews_link.click()

    reviews = []

    while True:
        try:
            review_elements = wait.until(EC.presence_of_all_elements_located((By.CSS_SELECTOR, ".imdb-user-review")))
            if not review_elements:
                break  # No review elements found, exit the loop
            
            for element in review_elements:
                review = {}
                # Extract review details
                review['Title'] = element.find_element(By.CSS_SELECTOR, ".lister-item-header a").text
                review['Year'] = element.find_element(By.CSS_SELECTOR, ".lister-item-header span").text.strip('()').split('–')[0].strip()
                review['ID'] = element.find_element(By.CSS_SELECTOR, ".lister-item-header a").get_attribute("href").split('/')[4]
                review['IMDBReviewID'] = element.get_attribute("data-review-id")
                review['Comment'] = element.find_element(By.CSS_SELECTOR, ".content > .text").text.strip()
                spoiler_warning_elements = element.find_elements(By.CSS_SELECTOR, ".spoiler-warning")
                review['Spoiler'] = len(spoiler_warning_elements) > 0
                # Get the media type using Trakt API
                media_type = get_media_type(review['ID'])
                if media_type:
                    review['Type'] = media_type
                else:
                    review['Type'] = 'unknown'

                reviews.append(review)

            try:
                # Check if "Next" link exists
                next_link = driver.find_element(By.CSS_SELECTOR, "a.next-page")
                if next_link.get_attribute("href") == "#":
                    break  # No more pages, exit the loop

                next_link.click()

                # Wait until the URL changes
                wait.until(EC.url_changes(current_url))
                
                # Refresh review_elements
                review_elements = wait.until(EC.presence_of_all_elements_located((By.CSS_SELECTOR, ".imdb-user-review")))

            except (NoSuchElementException, TimeoutException):
                break  # "Next" link not found or timed out waiting for the 'Next' link, exit the loop
        except Exception as e:
            print(f"Exception occurred: {e}")
            break

    imdb_reviews = reviews

    print('Processing IMDB Data Complete')
    
    return imdb_watchlist, imdb_ratings, imdb_reviews


import json
import requests
import time
import urllib.parse
try:
    from IMDBTraktSyncer import errorHandling as EH
except ImportError:
    import errorHandling as EH

def getTraktData():
    # Process Trakt Ratings and Comments
    print('Processing Trakt Data')

    response = EH.make_trakt_request('https://api.trakt.tv/users/me')
    json_data = json.loads(response.text)
    username_slug = json_data['ids']['slug']
    encoded_username = urllib.parse.quote(username_slug)
    
    # Get Trakt Watchlist Items
    response = EH.make_trakt_request(f'https://api.trakt.tv/users/{encoded_username}/watchlist?sort=added,asc')
    json_data = json.loads(response.text)

    trakt_watchlist = []

    for item in json_data:
        if item['type'] == 'movie':
            movie = item.get('movie')
            movie_id = movie.get('ids', {}).get('imdb')
            trakt_watchlist.append({'Title': movie.get('title'), 'Year': movie.get('year'), 'ID': movie_id, 'Type': 'movie'})
        elif item['type'] == 'show':
            show = item.get('show')
            show_id = show.get('ids', {}).get('imdb')
            trakt_watchlist.append({'Title': show.get('title'), 'Year': show.get('year'), 'ID': show_id, 'Type': 'show'})
        elif item['type'] == 'episode':
            show = item.get('show')
            show_title = show.get('title')
            episode = item.get('episode')
            episode_id = episode.get('ids', {}).get('imdb')
            episode_title = f'{show_title}: {episode.get("title")}'
            trakt_watchlist.append({'Title': episode_title, 'Year': episode.get('year'), 'ID': episode_id, 'Type': 'episode'})
    
    # Get Trakt Ratings
    response = EH.make_trakt_request(f'https://api.trakt.tv/users/{encoded_username}/ratings?sort=newest')
    json_data = json.loads(response.text)

    movie_ratings = []
    show_ratings = []
    episode_ratings = []

    for item in json_data:
        if item['type'] == 'movie':
            movie = item.get('movie')
            movie_id = movie.get('ids', {}).get('imdb')
            movie_ratings.append({'Title': movie.get('title'), 'Year': movie.get('year'), 'Rating': item.get('rating'), 'ID': movie_id, 'Type': 'movie'})
        elif item['type'] == 'show':
            show = item.get('show')
            show_id = show.get('ids', {}).get('imdb')
            show_ratings.append({'Title': show.get('title'), 'Year': show.get('year'), 'Rating': item.get('rating'), 'ID': show_id, 'Type': 'show'})
        elif item['type'] == 'episode':
            show = item.get('show')
            show_title = show.get('title')
            episode = item.get('episode')
            episode_id = episode.get('ids', {}).get('imdb')
            episode_title = f'{show_title}: {episode.get("title")}'
            episode_ratings.append({'Title': episode_title, 'Year': episode.get('year'), 'Rating': item.get('rating'), 'ID': episode_id, 'Type': 'episode'})

    trakt_ratings = movie_ratings + show_ratings + episode_ratings

    # Get Trakt Comments
    response = EH.make_trakt_request(f'https://api.trakt.tv/users/{encoded_username}/comments?sort=newest')
    json_data = json.loads(response.text)
    total_pages = response.headers.get('X-Pagination-Page-Count')
    trakt_comments = []

    for page in range(1, int(total_pages) + 1):
        response = EH.make_trakt_request(f'https://api.trakt.tv/users/{encoded_username}/comments', params={'page': page})
        json_data = json.loads(response.text)

        for comment in json_data:
            comment_type = comment['type']
            spoiler = comment.get('spoiler', False)

            if comment_type == 'movie':
                movie = comment.get('movie')
                show_movie_or_episode_title = movie.get('title')
                show_movie_or_episode_year = movie.get('year')
                show_movie_or_episode_imdb_id = movie.get('ids', {}).get('imdb')
            elif comment_type == 'episode':
                show = comment.get('show')
                episode = comment.get('episode')
                show_movie_or_episode_title = f"{show.get('title')}: {episode.get('title')}"
                show_movie_or_episode_year = show.get('year')
                show_movie_or_episode_imdb_id = episode.get('ids', {}).get('imdb')
            elif comment_type == 'show':
                show = comment.get('show')
                show_movie_or_episode_title = show.get('title')
                show_movie_or_episode_year = show.get('year')
                show_movie_or_episode_imdb_id = show.get('ids', {}).get('imdb')
            elif comment_type == 'season':
                show = comment.get('show')
                season = comment.get('season')
                show_movie_or_episode_title = f"{show.get('title')}: Season {season.get('number')}"
                show_movie_or_episode_year = None
                show_movie_or_episode_imdb_id = None
            else:
                show_movie_or_episode_title = None
                show_movie_or_episode_year = None
                show_movie_or_episode_imdb_id = None

            comment_info = comment['comment']
            trakt_comment_id = comment_info.get('id')
            trakt_comment = comment_info.get('comment')

            trakt_comments.append({
                'Title': show_movie_or_episode_title,
                'Year': show_movie_or_episode_year,
                'ID': show_movie_or_episode_imdb_id,
                'TraktCommentID': trakt_comment_id,
                'Comment': trakt_comment,
                'Spoiler': spoiler,
                'Type': comment_type
            })

    print('Processing Trakt Data Complete')

    return trakt_watchlist, trakt_ratings, trakt_comments